/*****************************************************
CodevisionAVR
Chip type               : ATtiny13
AVR Core Clock frequency: 1,2 MHz
*****************************************************/

#include <tiny13.h>
#include <delay.h>


// Portpin fr Tonausgabe
#define P_TON   PORTB.4
#define DP_TON  DDRB.4

// Periodendauer eines Tons in s
#define DELAY_TON_C1  3788
#define DELAY_TON_D1  3367
#define DELAY_TON_E1  3030
#define DELAY_TON_F1  2841
#define DELAY_TON_G1  2525
#define DELAY_TON_A1  2273
#define DELAY_TON_H1  2020
#define DELAY_TON_C2  1894

enum { TON_C1, TON_D1, TON_E1, TON_F1, TON_G1, TON_A1, TON_H1, TON_C2 };

const flash unsigned int tone_period[] = 
{ 
   DELAY_TON_C1, DELAY_TON_D1, DELAY_TON_E1, DELAY_TON_F1, 
   DELAY_TON_G1, DELAY_TON_A1, DELAY_TON_H1, DELAY_TON_C2 
};

#define TON_PAUSE  10      // Pause zwischen Tnen in ms

#define DELAY_NOTE_32  30  // Dauer einer 1/32-Note in ms
#define NOTE_32  1
#define NOTE_16 (NOTE_32*2)
#define NOTE_8  (NOTE_32*4)
#define NOTE_4  (NOTE_32*8)
#define NOTE_2  (NOTE_32*16)
#define NOTE_1  (NOTE_32*32)


typedef struct
{
   unsigned char tone;
   unsigned char duration;
} T_NOTES;

   
T_NOTES flash jingleBells[] =
{
       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_2,
       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_2,
       TON_E1, NOTE_4,
       TON_G1, NOTE_4,
       TON_C1, NOTE_4,
       TON_D1, NOTE_4,
       TON_E1, NOTE_2,
      
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_8,
       TON_E1, NOTE_8,
       TON_E1, NOTE_4,
       TON_D1, NOTE_4,
       TON_D1, NOTE_4,
       TON_E1, NOTE_4,
       TON_D1, NOTE_2,
       TON_G1, NOTE_2,

       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_2,
       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_2,
       TON_E1, NOTE_4,
       TON_G1, NOTE_4,
       TON_C1, NOTE_4,
       TON_D1, NOTE_4,
       TON_E1, NOTE_2,
      
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_F1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_4,
       TON_E1, NOTE_8,
       TON_E1, NOTE_8,
       TON_G1, NOTE_4,
       TON_G1, NOTE_4,
       TON_F1, NOTE_4,
       TON_D1, NOTE_4,
       TON_C1, NOTE_2
};       



void playTone(unsigned char tone, unsigned char duration)
{
   unsigned int repeat;
   
   repeat = (unsigned long)duration * DELAY_NOTE_32 * 1000 / (tone_period[tone]);

   switch(tone)
   {
      case TON_C1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_C1/2);
            P_TON = 0;
            delay_us(DELAY_TON_C1/2);
            repeat--;
         }        
      break;

      case TON_D1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_D1/2);
            P_TON = 0;
            delay_us(DELAY_TON_D1/2);
            repeat--;
         }        
      break;
         
      case TON_E1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_E1/2);
            P_TON = 0;
            delay_us(DELAY_TON_E1/2);
            repeat--;
         }        
      break;

      case TON_F1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_F1/2);
            P_TON = 0;
            delay_us(DELAY_TON_F1/2);
            repeat--;
         }        
      break;

      case TON_G1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_G1/2);
            P_TON = 0;
            delay_us(DELAY_TON_G1/2);
            repeat--;
         }        
      break;

      case TON_A1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_A1/2);
            P_TON = 0;
            delay_us(DELAY_TON_A1/2);
            repeat--;
         }        
      break;

      case TON_H1:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_H1/2);
            P_TON = 0;
            delay_us(DELAY_TON_H1/2);
            repeat--;
         }        
      break;

      case TON_C2:   
         while(repeat > 0)
         {                  
            P_TON = 1;
            delay_us(DELAY_TON_C2/2);
            P_TON = 0;
            delay_us(DELAY_TON_C2/2);
            repeat--;
         }        
      break;       
   }
      
   delay_ms(10);
}


void main(void)
{
   unsigned int i;

   // Analog Comparator initialization
   // Analog Comparator: Off
   // Analog Comparator Input Capture by Timer/Counter 1: Off
   ACSR=0x80;
   ADCSRB=0x00;

   P_TON = 0;
   DP_TON = 1;


   while(1)
   {
      for(i = 0; i < sizeof(jingleBells) / sizeof(T_NOTES); i++)
      {
         playTone(jingleBells[i].tone, jingleBells[i].duration);
      }
      delay_ms(2000);
   }


}



